#include "uart.h"
#include <avr/interrupt.h>


unsigned char Sio1RecBuf[SIORECBUFSIZE];
unsigned char Sio1SendBuf[SIOSENDBUFSIZE];
short Sio1RecBufWritePtr;
short Sio1RecBufReadPtr;
short Sio1SendBufPtr;
short Sio1SendBufEndPtr;
SioBufStatusStruct volatile Sio1BufStatus;



/****************************************************************
* Function  : Sio1FlushBuffers                                  *
*---------------------------------------------------------------*
* Purpose   : initializes SIO1 buffers and pointers             *
*---------------------------------------------------------------*
* Scope     : file                                              *
*---------------------------------------------------------------*
* Input     : none                                              *
*---------------------------------------------------------------*
* Output    : none                                              *
****************************************************************/
unsigned char Sio1FlushBuffers(void)  {                   
  /* setup status information */
  Sio1RecBufWritePtr         = 0;
  Sio1RecBufReadPtr          = 0;
  Sio1BufStatus.Bytes        = 0;
  Sio1BufStatus.NewRecByte   = 0;
  return 0;
}


ISR(USART1_RX_vect)  {
  unsigned char dummy;         
  if (!(Sio1BufStatus.Bytes >= SIORECBUFSIZE))  {
    Sio1RecBuf[Sio1RecBufWritePtr++] = UDR1;
    Sio1BufStatus.Bytes++;
    if (Sio1RecBufWritePtr == SIORECBUFSIZE)
      Sio1RecBufWritePtr = 0;
    Sio1BufStatus.NewRecByte = 1;
  }
  else dummy = UDR1;
}

/*
 * Initialize the UART baud rate baud
 */
void  uart_init(unsigned long baud)  {
  Sio1FlushBuffers();
  unsigned int v = (F_CPU / (16UL * baud)) - 1;
  
  UBRR1H = (unsigned char)(v>>8);
  UBRR1L = (unsigned char)v;
/* Enable receiver and transmitter */
  UCSR1B = _BV(TXEN) | _BV(RXEN) | _BV(RXCIE); /* tx/rx enable */
/* Set frame format: 8data, 1stop bit */
  UCSR1C = (1<<USBS)|(3<<UCSZ0);  
}


/****************************************************************
* Function  : Sio1GetRecByte                                    *
*---------------------------------------------------------------*
* Purpose   : copies one byte of receive buffer to data pointer.*
*---------------------------------------------------------------*
* Scope     : global, API                                       *
*---------------------------------------------------------------*
* Input     : *Data  : data pointer to destination              *
*---------------------------------------------------------------*
* Output    : 1: byte has been copied, 0: rec buffer empty      *
****************************************************************/
unsigned char Sio1GetRecByte(unsigned char *Data)  {              
  if (!Sio1BufStatus.Bytes)
    return 0;
  *Data = Sio1RecBuf[Sio1RecBufReadPtr++];
  if  (Sio1RecBufReadPtr == SIORECBUFSIZE)
    Sio1RecBufReadPtr =  0;
  Sio1BufStatus.Bytes--;
  return 1;
}
 
 
/****************************************************************
* Function  : Sio1IntTxD                                        *
*---------------------------------------------------------------*
* Purpose   : interrupt service funcitons for byte transmit,    *
*             to be referenced in interrupt vector table!!      *
*---------------------------------------------------------------*
* Scope     : global, API                                       *
*---------------------------------------------------------------*
* Input     : none                                              *
*---------------------------------------------------------------*
* Output    : none                                              *
****************************************************************/
ISR(USART1_TX_vect)  {
  if(Sio1SendBufPtr == Sio1SendBufEndPtr)
    Sio1BufStatus.Sending = 0;
  else UDR1 = Sio1SendBuf[Sio1SendBufPtr++];
}

/****************************************************************
* Function  : Sio1Transmit                                      *
*---------------------------------------------------------------*
* Purpose   : copies a block of data to an ouput buffer and     *
*             starts sending its content to a serial channel,   *
*             work interrupt driven in background               *
*---------------------------------------------------------------*
* Scope     : global, API                                       *
*---------------------------------------------------------------*
* Input     : ToSend : byte pointer to the block to be send     *
*             Count  : number of bytes to send                  *
*---------------------------------------------------------------*
* Output    : none                                              *
****************************************************************/
void Sio1Transmit(unsigned char* ToSend, short Count)  {
  short idx;
  if (!Count)  {
    Sio1BufStatus.Sending   = 0;
    return;
  }
  Sio1BufStatus.Sending = 1;
  Sio1SendBufEndPtr = Count;
  idx = 0;
  while(Count--)
    Sio1SendBuf[idx++] = *ToSend++;
  Sio1SendBufPtr      = 1;
  UDR1 = Sio1SendBuf[0];
}
 

/*
 * Send character c down the UART Tx, wait until tx holding register
 * is empty.
 */
int  uart_putchar(int c)  {
  loop_until_bit_is_set(UCSR1A, UDRE);
  UDR1 = c;
  return 0;
}

int uart_putc(unsigned char c) {

	loop_until_bit_is_set(UCSR1A, UDRE);
	UDR1 = c;
	return 0;
}

void uart_puts(char *s) {

	while (*s) {
	
		uart_putc(*s);
		s++;
	
	}
}





/*
 * Receive a character from the UART Rx.
 *
 */
int  uart_getchar(void)  {
  uint8_t c;
  loop_until_bit_is_set(UCSR1A, RXC);
  c = UDR1;
  return c;
}

int uart_keyhit(void)  {
  if(bit_is_set(UCSR1A,RXC))
    return 1;
  else
    return 0;
}
